# Spectra

A tool for automated translation of specification to a structured data. It consists of more partial projects (currently only one, more will follow): 
* *T-Props Checker* - a tool for checking temporal properties of finite runs of programs in C/C++.

## Getting Started

The tool is based on a C-language program, and due to usage of regular expressions it might fail to be compiled using older gcc than 4.9. However, there are no other software requirements for the compilation. All other specific needs (e.g. your program modifications, needed files, etc.) for each subtool usage are stated in their description below. The command `make` creates an executable named *spectra*. Some example usages can be found in the [tests](tests) folder.
 
## Running the tests

The tests can be run from the base folder or the [tests](tests) folder using command `make test`. This compiles the project and then executes tests for each implemented tool. In the tests folder there is a folder for each of the tools where a shell script for running tests for this particular tool can be found. These scripts are called by the `make test` command and described in the [tests/README.md](tests/README.md).

## Deployment

When the *spectra* executable is build successfully, it can be used to perform the tasks implemented by it's subtools as follows.

### T-Props Checker
![scheme](doc/T-Props_Checker/TPC-scheme.jpg "Logo Title Text 1")

To check the temporal properties of your program you'll need:
* a specification of your program's expected behaviour using ptLTL formulae (explained e.g. in this [course](https://ti.tuwien.ac.at/cps/teaching/courses/runtime-verification/rvcourse20162.pdf)), written in a file, formated according to [this](doc/T-Props_Checker/InputFileSpec.md) specification, some examples can be found in the [tests](tests/T-Props-Checker/specifications) folder, and
* all variables or functions, values of which appear in any part of the specification, which are not defined there, declared in a *.h* file.

Then just simply run the project as 
```
./spectra --tpc -s path/to/specification.spec -d path/to/defs.h
```
where --tpc selects usage of the T-Props Checker tool and -s and -d specifies path to the two needed files. The program will generate two files named *monitor.h*, which needs to be included in the checked source files, and *libspectra.a*. This library can be then linked to any C/C++ project, by adding
* '-I.' to CXXFLAGS, denoting that the object files inside the library are searched in the root folder,
* '-L.' to LDFLAGS, telling the compiler where to search for libraries - if the file *libspectra.a* was placed in some other folder, that the one your program is compiled from, you need to replace the '.' with corresponding path to the containing folder.
* '-lspectra' to LDLIBS, for linking the library *libspectra.a* to your program.

To be able to verify a program
1. include *monitor.h* to any of your original program files where there the current execution state can be verified - if including into C++ program, you need to wrap the #include line in extern "C" {...}
2. mark appropriate locations with either
    * a comment '// T-Props-Checker verify'
    * a call of the 'monitorVerify();' function declared in *monitor.h* file
    
3. run the spectra program to generate the monitor and - if needed - replace the comments by function calls and check for including the *monitor.h* file (the previous version will be kept, renamed to 'the/same/name.old')
4. link the static library to your project by adding the above mentioned flags to your makefile/compile command

If suitable, you can use the *monitorVerify()* boolean return value: 
* *true* if current state of the program corresponds with the specification, 
* *false* otherwise. If *false* is returned, there will be a message describing which part(s) of the formuae were violated printed to stderr.
You can also reset the monitor verification by calling function *reset()*, e.g. when you reinitialized the checked program, to make the monitor forget about the previous errors.

The tests script [tpc-test.sh](tests/T-Props-Checker/tpc-test.sh) could be useful as an example usage of compilation of the system under test.

## Authors

* **Petra Sečkařová** - [T-Props Checker](src/T-Props_Checker)
