# Formulae file format specification

The input file of [T-Props
Checker](https://pajda.fit.vutbr.cz/testos/spectra#t-props-checker) can have
from 1 to 3 parts as described in the following three sections of this
document.

## Listing of formulae to be monitored

The formulae section is reqired. The content of this section are formulae
allowed to be constructed as

    F := varName | (!F) | (-> F F) | (|| F F) | (&& F F) | (^ F F)
        (propositional operators)
    
    (G F) | (P F) | (L F) | (Ss F F) | (Sw F F)
        (past-time operators)

    (U F) | (D F) | (Is F F) | (Iw F F)
        (monitoring operators)

where the propositional operators `&&`, `||` and `^` are allowed to have two or
more operators and varNames correspond with C language restrictions. 

Assuming, that s_n is a current state of the monitored run, which is given by
trace t&nbsp;=&nbsp;s_1,&nbsp;s_2,&nbsp;s_3,&nbsp;...,&nbsp;s_n, the
description of the operators would be as listed in the table below (sematics is
taken from *K. Havelund, G. Roşu: Efficient monitoring of safety properties*).

|Operator |Name           |Meaning                                                                |
|:-------:|:-------------:|-----------------------------------------------------------------------|
|D        |Down	          |t_n-1 \|= F and **not** t \|= F                                        |
|G		  |Globally       |t \|= F for all s_i, 1 ≤ i ≤ n                                         |
|Is       |Strong interval|t_j \|= F1 for some 1 ≤ j ≤ n and **not** t_i \| = F2 for all j ≤ i ≤ n| 
|Iw       |Weak interval  |t \|= (Is F1 , F2 ) or t \|= (!F2)                                     |
|L        |Last           |t' \|= F, where t' = t_n−1 if n > 1 and t' = t if n = 1                |
|P        |Previously     |t \|= F for any s_i, 1 ≤ i ≤ n                                         |
|Ss       |Strong since   |t_j \|= F2 for some 1 ≤ j ≤ n and t_i \|= F1 for all j < i ≤ n         |
|Sw       |Weak since     |t \|= (Ss F1 F2) or t \|= (A F1)                                       |
|U        |Up		      |t \|= F and **not** t_n−1 \| = F                                       |
|&&       |And		      |logical and of all arguments is true                                   |
|\|\|     |Or 		      |logical or of all arguments is true                                    |
|^        |Xor            |logical xor of all arguments is true                                   |
|!        |Not		      |F is false                                                             |
|->       |Implies		  |t \|= (!F1) or t \|= F2                                                |

There can be any number of separate formulae in the formulae listing section of
the file, separated by any **white characters**, that will be considered as a
connected by &&. The boolean operators &&, \|\| and ^ can have 2 or more
operands, Is, Iw, Ss, Sw and -> are allowed to have exactly two and D, G, L, P
and ! can have only one.

## Definitions of variables used in listed formulae

This section of the file has to be included, if not all of the state variables
are declared as type *bool* in the definitions header file. It begins with a
line

```
DEFINITIONS:
```

followed by definitions of the variables (every varName not defined in the
header file) used in previous formulae, formated as *name : expression*. To
define value of a variable, anything defined in the definitions header file can
be used and combined with any constants or functions, e.g.

```
a : i < 3
z : isCorrect(d)
x : true
v42 : (bool)((i*3)%2)
```

The only limitation is the type of the resulting value -- it always has to be
the **bool** type.

## List of source files where instrumetation comments can be placed

The last section is optional and is separated from the previous content by line 

```
SOURCES:
```

On each line there can be a relative path to a source file (relative to the
specification file). These files will be searched for a comment

```
// T-Props Checker verify
```

which will be replaced by a call of the monitorVerify() function from the
generated monitor. If the files do not include the monitor.h file, a line(s)
with

```
[extern "C" {]
#include "monitor.h"
[}]
```

are added after the last line with #include. The *extern* part is used only for
C++ source files.
