//
// Created by seckarovap on 4/2/19.
//

#include "formulaAnalyserTest.h"

#define SPEC_DIR "T-Props-Checker/specifications/unit-tests/"

TEST_F(formulaAnalyserTest, Parse_no_formula) {
    // no problem if an empty file given as a specification
    ASSERT_TRUE(analyser.parseFormula(SPEC_DIR "no_formula.spec"));
}

TEST_F(formulaAnalyserTest, Parse_empty_parentheses) {
    // can't parse () in formulae definitions
    ASSERT_FALSE(analyser.parseFormula(SPEC_DIR "empty_parentheses.spec"));
}

TEST_F(formulaAnalyserTest, Parse_globally_true) {
    // (G true)
    ASSERT_TRUE(analyser.parseFormula(SPEC_DIR "globally_true.spec"));
}

TEST_F(formulaAnalyserTest, Parse_globally_true_unclosed) {
    // (G true
    ASSERT_FALSE(analyser.parseFormula(SPEC_DIR "globally_true_missing_).spec"));
}

TEST_F(formulaAnalyserTest, Parse_globally_true_too_any_closing_parentheses) {
    // (G true))
    ASSERT_FALSE(analyser.parseFormula(SPEC_DIR "globally_true_one_)_too_much.spec"));
}

TEST_F(formulaAnalyserTest, Parse_parentheses_for_not) {
    // !(P (!(&& var1 !(G (!var2)))))
    // !(Ss !(var1) (!(P (!var2)))))
    ASSERT_TRUE(analyser.parseFormula(SPEC_DIR "parentheses_for_not.spec"));
}

TEST_F(formulaAnalyserTest, Parse_atom_with_invalid_definition) {
    ASSERT_FALSE(analyser.parseFormula(SPEC_DIR "atom_with_invalid_definition.spec"));
}

TEST_F(formulaAnalyserTest, Parse_atom_with_more_definitions_than_needed) {
    ASSERT_TRUE(analyser.parseFormula(SPEC_DIR "atom_with_more_definitions_than_needed.spec"));
}

TEST_F(formulaAnalyserTest, Parse_atom_with_valid_definition) {
    ASSERT_TRUE(analyser.parseFormula(SPEC_DIR "atom_with_valid_definition.spec"));
}

TEST_F(formulaAnalyserTest, Parse_incomplete_formula) {
    ASSERT_FALSE(analyser.parseFormula(SPEC_DIR "incomplete_formula.spec"));
}

TEST_F(formulaAnalyserTest, Parse_invalid_operator) {
    ASSERT_FALSE(analyser.parseFormula(SPEC_DIR "invalid_operator.spec"));
}

TEST_F(formulaAnalyserTest, Parse_not_spacing_variants) {
    ASSERT_TRUE(analyser.parseFormula(SPEC_DIR "not_spacing_variants.spec"));
}

TEST_F(formulaAnalyserTest, Parse_too_many_operands_parentheses) {
    ASSERT_FALSE(analyser.parseFormula(SPEC_DIR "too_many_operands_parentheses.spec"));
}

TEST_F(formulaAnalyserTest, Parse_too_many_operands) {
    ASSERT_FALSE(analyser.parseFormula(SPEC_DIR "too_many_operands.spec"));
}

TEST_F(formulaAnalyserTest, Parse_invalid_path_to_spec) {
    ASSERT_FALSE(analyser.parseFormula(SPEC_DIR "none"));
}

TEST_F(formulaAnalyserTest, Parse_demanding_atom_immediate_closing) {
    ASSERT_FALSE(analyser.parseFormula(SPEC_DIR "demanding_atom_immediate_closing.spec"));
}

TEST_F(formulaAnalyserTest, Parse_demanding_atom_immediate_closing1) {
    ASSERT_FALSE(analyser.parseFormula(SPEC_DIR "demanding_atom_immediate_closing1.spec"));
}

TEST_F(formulaAnalyserTest, Invalid_path) {
    ASSERT_FALSE(analyser.parseFormula("invalid/path"));
}

void operandTest(formulaAnalyser analyser, bool canAddOperand[], bool isFullySet[], int cnt, OpType operation) {
    std::stack<formula> formulaStack;
    std::vector<int> activeFormulae;
    formula *currentFormula = nullptr;
    int nextId = 0;

    analyser.addNewFormula(formulaStack, activeFormulae, nextId, currentFormula);
    currentFormula->setOperation(operation);


    std::string name = "var";
    std::string filename = "file";

    for (int i = 0; i < cnt ; i++) {
        ASSERT_EQ(isFullySet[i], currentFormula->isFullySet());
        ASSERT_EQ(canAddOperand[i],
                  analyser.generateAtom(name, nextId, currentFormula, formulaStack, activeFormulae, 1, 1, filename));
    }
}

TEST_F(formulaAnalyserTest, Operands_count_globally) {

    bool canAddOperand[] = {true, false, false};
    bool isFullySet[] = {false, true, true};

    operandTest(analyser, canAddOperand, isFullySet, 3, GLOBALLY);
}

TEST_F(formulaAnalyserTest, Operands_count_last) {

    bool canAddOperand[] = {true, false, false};
    bool isFullySet[] = {false, true, true};

    operandTest(analyser, canAddOperand, isFullySet, 3, LAST);

}
TEST_F(formulaAnalyserTest, Operands_count_previously) {

    bool canAddOperand[] = {true, false, false};
    bool isFullySet[] = {false, true, true};

    operandTest(analyser, canAddOperand, isFullySet, 3, PREVIOUSLY);

}

TEST_F(formulaAnalyserTest, Operands_count_Ssince) {

    bool canAddOperand[] = {true, true, false, false};
    bool isFullySet[] = {false, false, true, true};

    operandTest(analyser, canAddOperand, isFullySet, 4, S_SINCE);

}

TEST_F(formulaAnalyserTest, Operands_count_Wsince) {

    bool canAddOperand[] = {true, true, false, false};
    bool isFullySet[] = {false, false, true, true};

    operandTest(analyser, canAddOperand, isFullySet, 4, W_SINCE);

}

TEST_F(formulaAnalyserTest, Operands_count_up) {

    bool canAddOperand[] = {true, false, false};
    bool isFullySet[] = {false, true, true};

    operandTest(analyser, canAddOperand, isFullySet, 3, UP);
}

TEST_F(formulaAnalyserTest, Operands_count_down) {

    bool canAddOperand[] = {true, false, false};
    bool isFullySet[] = {false, true, true};

    operandTest(analyser, canAddOperand, isFullySet, 3, DOWN);
}

TEST_F(formulaAnalyserTest, Operands_count_Sinterval) {

    bool canAddOperand[] = {true, true, false, false};
    bool isFullySet[] = {false, false, true, true};

    operandTest(analyser, canAddOperand, isFullySet, 4, S_INTERVAL);
}

TEST_F(formulaAnalyserTest, Operands_count_Winterval) {

    bool canAddOperand[] = {true, true, false, false};
    bool isFullySet[] = {false, false, true, true};

    operandTest(analyser, canAddOperand, isFullySet, 4,W_INTERVAL);
}

TEST_F(formulaAnalyserTest, Operands_count_And) {

    bool canAddOperand[] = {true, true, true, true};
    bool isFullySet[] = {false, false, true, true};

    operandTest(analyser, canAddOperand, isFullySet, 4, AND);
}

TEST_F(formulaAnalyserTest, Operands_count_Or) {

    bool canAddOperand[] = {true, true, true, true};
    bool isFullySet[] = {false, false, true, true};

    operandTest(analyser, canAddOperand, isFullySet, 4, OR);
}

TEST_F(formulaAnalyserTest, Operands_count_Xor) {

    bool canAddOperand[] = {true, true, true, true};
    bool isFullySet[] = {false, false, true, true};

    operandTest(analyser, canAddOperand, isFullySet, 4, XOR);
}

TEST_F(formulaAnalyserTest, Operands_count_Not) {

    bool canAddOperand[] =  {true, false, false};
    bool isFullySet[] = {false, true, true};

    operandTest(analyser, canAddOperand, isFullySet, 3, NOT);
}

TEST_F(formulaAnalyserTest, Operands_count_Imply) {

    bool canAddOperand[] = {true, true, false, false};
    bool isFullySet[] = {false, false, true, true};

    operandTest(analyser, canAddOperand, isFullySet, 4, IMPLY);
}


TEST_F(formulaAnalyserTest, Operands_count_Atom) {

    std::stack<formula> formulaStack;
    std::vector<int> activeFormulae;
    formula *currentFormula = nullptr;
    int nextId = 0;

    analyser.addNewFormula(formulaStack, activeFormulae, nextId, currentFormula);
    currentFormula->setOperation(ATOM);

    bool canAddOperand[] =  {false, false};
    bool isFullySet[] = {false, false};

    std::string name = "var";
    std::string filename = "file";

    for (int i = 0; i < 2 ; i++) {
        ASSERT_EQ(isFullySet[i], currentFormula->isFullySet());
        ASSERT_EQ(canAddOperand[i],
                  analyser.generateAtom(name, nextId, currentFormula, formulaStack, activeFormulae, 1, 1, filename));
    }

    currentFormula->setContent(name);
    ASSERT_TRUE(currentFormula->isFullySet());
}
