//
// Created by seckarovap on 5/12/19.
//

#include "monitorCreatorTest.h"

#define SPEC_DIR "T-Props-Checker/specifications/unit-tests/"
#define DEFS "T-Props-Checker/definitions/unit-tests/defs.h"
#define SUT_DIR "T-Props-Checker/SUT/unit-tests/"

enum S {INIT, PREPARE, VERIFY, DONE};


void monitorCreatorTest::testOperator(std::string spec_name, std::string prep, std::string verify) {
    mCreator.setFormulaFile(spec_name);
    ASSERT_TRUE(mCreator.printMonitor(DEFS));

    std::ifstream monitor(GENERATED_FILES_NAME ".c");
    std::string line;
    S state = INIT;
    bool prep_found = false, verify_found = false;

    if (monitor.is_open())
        while (std::getline(monitor, line) && state != DONE) {
            switch (state) {
                case INIT:
                    if (line.find("void prepare()") != std::string::npos)
                        state = PREPARE;
                    break;
                case PREPARE:
                    if (line.find("bool monitorVerify()") != std::string::npos)
                        state = VERIFY;
                    else {
                        // the root formula?
                        if (line.find("M.pre[0]") != std::string::npos){
                            prep_found = true;
                            ASSERT_NE(line.find(prep),std::string::npos);
                        }
                    }
                    break;
                default:
                    // the root formula?
                    if (line.find("M.now[0] = ") != std::string::npos){
                        verify_found = true;
                        ASSERT_NE(line.find(verify),std::string::npos);
                        state = DONE;
                    }
                    break;
            }
        }

    ASSERT_TRUE(prep_found && verify_found);
    monitor.close();

    //! TEARDOWN
    remove(GENERATED_FILES_NAME ".*");
}


TEST_F(monitorCreatorTest, Last) {
    testOperator(SPEC_DIR "last.spec", "M.pre[1]", "M.pre[1]");
}

TEST_F(monitorCreatorTest, Previously) {
    testOperator(SPEC_DIR "previously.spec", "M.pre[1]", "M.now[1] || M.pre[0]");
}

TEST_F(monitorCreatorTest, Globally) {
    testOperator(SPEC_DIR "globally.spec", "M.pre[1]", "M.now[1] && M.pre[0]");
}

TEST_F(monitorCreatorTest, Up) {
    testOperator(SPEC_DIR "up.spec", "false", "!M.pre[1] && M.now[1]");
}

TEST_F(monitorCreatorTest, Down) {
    testOperator(SPEC_DIR "down.spec", "false", "M.pre[1] && !M.now[1]");
}

TEST_F(monitorCreatorTest, Not) {
    testOperator(SPEC_DIR "not.spec", "!M.pre[1]", "!M.now[1]");
}

TEST_F(monitorCreatorTest, Strong_since) {
    testOperator(SPEC_DIR "strong_since.spec", "M.pre[2]", "M.now[2] || (M.now[1] && M.pre[0])");
}

TEST_F(monitorCreatorTest, Weak_since) {
    testOperator(SPEC_DIR "weak_since.spec", "M.pre[2] || M.pre[1]", "M.now[2] || (M.now[1] && M.pre[0])");
}

TEST_F(monitorCreatorTest, Strong_interval) {
    testOperator(SPEC_DIR "strong_interval.spec", "!M.pre[2] && M.pre[1]", "!M.now[2] && (M.pre[0] || M.now[1])");
}

TEST_F(monitorCreatorTest, Weak_interval) {
    testOperator(SPEC_DIR "weak_interval.spec", "!M.pre[2]", "!M.now[2] && (M.pre[0] || M.now[1])");
}

TEST_F(monitorCreatorTest, And) {
    testOperator(SPEC_DIR "and.spec", "M.pre[1] && M.pre[2]", "M.now[1] && M.now[2]");
}

TEST_F(monitorCreatorTest, Or) {
    testOperator(SPEC_DIR "or.spec", "M.pre[1] || M.pre[2]", "M.now[1] || M.now[2]");
}

TEST_F(monitorCreatorTest, Xor) {
    testOperator(SPEC_DIR "xor.spec", "M.pre[1] ^ M.pre[2]", "M.now[1] ^ M.now[2]");
}

TEST_F(monitorCreatorTest, Imply) {
    testOperator(SPEC_DIR "imply.spec", "!M.pre[1] || M.pre[2]", "!M.now[1] || M.now[2]");
}