//
// Created by seckarovap on 4/2/19.
//

#include "monitorCreatorTest.h"

#define SPEC_DIR "T-Props-Checker/specifications/unit-tests/"
#define SUT_DIR "T-Props-Checker/SUT/unit-tests/"

TEST_F(monitorCreatorTest, FileSize) {
    ASSERT_EQ(fileSize(SPEC_DIR "no_formula.spec"), 0);
    ASSERT_EQ(fileSize(SPEC_DIR "invalid_operator.spec"), 14);
    ASSERT_EQ(fileSize(SPEC_DIR "atom_with_more_definitions_than_needed.spec"), 54);
}

TEST_F(monitorCreatorTest, Replace_instrumentation_comments_one_file) {
    mCreator.setFormulaFile("given.spec");
    mCreator.replaceLabelsInFileWithMonitorCalls(SUT_DIR "instrumentation_comments.cpp");
    mCreator.replaceLabelsInFileWithMonitorCalls(SUT_DIR "no_instrumentation_comments.cpp");

    std::ifstream src(SUT_DIR "instrumentation_comments.cpp");
    std::string line;

    if (src.is_open())
        while (std::getline(src, line)) {
            //! all comments to be replaced in the test src are exactly in this format
            ASSERT_EQ(line.find("// T-Props Checker verify"), std::string::npos);
        }

    src.open(SUT_DIR "no_instrumentation_comments.cpp");

    if (src.is_open())
        while (std::getline(src, line)) {
            //! all comments to be replaced in the test src are exactly in this format
            ASSERT_EQ(line.find("// T-Props Checker verify"), std::string::npos);
        }
    src.close();

    src.open(SUT_DIR "no_instrumentation_comments.cpp.old");
    ASSERT_FALSE(src.is_open());

    //! TEARDOWN
    std::ifstream srcOld(SUT_DIR "instrumentation_comments.cpp.old");
    std::ofstream overwrite(SUT_DIR "instrumentation_comments.cpp");

    if (srcOld.is_open())
        while (std::getline(srcOld, line)) {
            overwrite << line << std::endl;
        }

    srcOld.close();
    overwrite.close();

    remove(SUT_DIR "instrumentation_comments.cpp.old");
}


TEST_F(monitorCreatorTest, Replace_instrumentation_comments_src_list) {
    mCreator.setFormulaFile("given.spec");

    std::vector<std::string> sources = {SUT_DIR "instrumentation_comments.cpp", SUT_DIR "no_instrumentation_comments.cpp"};

    mCreator.replaceInstrumentationComments(sources);

    std::ifstream src(SUT_DIR "instrumentation_comments.cpp");
    std::string line;

    if (src.is_open())
        while (std::getline(src, line)) {
            //! all comments to be replaced in the test src are exactly in this format
            ASSERT_EQ(line.find("// T-Props Checker verify"), std::string::npos);
        }

    src.open(SUT_DIR "no_instrumentation_comments.cpp");

    if (src.is_open())
        while (std::getline(src, line)) {
            //! all comments to be replaced in the test src are exactly in this format
            ASSERT_EQ(line.find("// T-Props Checker verify"), std::string::npos);
        }
    src.close();

    src.open(SUT_DIR "no_instrumentation_comments.cpp.old");
    ASSERT_FALSE(src.is_open());

    //! TEARDOWN
    std::ifstream srcOld(SUT_DIR "instrumentation_comments.cpp.old");
    std::ofstream overwrite(SUT_DIR "instrumentation_comments.cpp");

    if (srcOld.is_open())
        while (std::getline(srcOld, line)) {
            overwrite << line << std::endl;
        }

    srcOld.close();
    overwrite.close();

    remove(SUT_DIR "instrumentation_comments.cpp.old");
}

TEST_F(monitorCreatorTest, GetCommentForGeneratedFiles) {
    mCreator.setFormulaFile("given.spec");
    std::string comment = mCreator.getCommentForGeneratedFiles("given/defs.h");

    ASSERT_NE(comment.find("Generated by monitorCreator class of T-Props Checker."), std::string::npos);
    ASSERT_NE(comment.find("Command: ./spectra --tpc"), std::string::npos);
    ASSERT_NE(comment.find("-s given.spec"), std::string::npos);
    ASSERT_NE(comment.find("-d given/defs.h"), std::string::npos);
}


TEST_F(monitorCreatorTest, PrintVarsEnum) {
    std::string filename = "tmp";
    FILE* outfile = fopen(filename.c_str(),"w");

    std::set<std::string> varNames = {"var1", "var2", "var3"};
    std::string expectedOutput = "enum VarNames { var1, var2, var3 };";

    mCreator.setFormulaFile("given.spec");
    mCreator.printVarNamesEnum(varNames,outfile);

    fclose(outfile);

    std::ifstream file(filename);
    std::string content;
    content.reserve(static_cast<unsigned long>(fileSize(filename.c_str())));

    if (file.is_open()) {
        content.assign((std::istreambuf_iterator<char>(file)),
                       std::istreambuf_iterator<char>());

        ASSERT_NE(content.find(expectedOutput), std::string::npos);
    }

    file.close();
    remove(filename.c_str());
}



TEST_F(monitorCreatorTest, GenerateHeaderFile) {
    std::string filename = GENERATED_FILES_NAME ".h";
    FILE* outfile = fopen(filename.c_str(),"w");

    std::string neededContent =  "#ifdef __cplusplus\n"\
                "extern \"C\" {\n#endif\n\n"\
                "   bool monitorVerify();\n\n"\
                "#ifdef __cplusplus\n}\n#endif";

    mCreator.setFormulaFile("given.spec");
    mCreator.generateHeaderFile("defs.h", "");

    fclose(outfile);

    std::ifstream file(filename);
    std::string content;
    content.reserve(static_cast<unsigned long>(fileSize(filename.c_str())));

    if (file.is_open()) {
        content.assign((std::istreambuf_iterator<char>(file)),
                       std::istreambuf_iterator<char>());

        ASSERT_NE(content.find("#include \"defs.h\""), std::string::npos);
        ASSERT_NE(content.find(neededContent), std::string::npos);
    }

    file.close();
    remove(filename.c_str());
}

monitorCreatorTest::monitorCreatorTest() = default;
